//----------------------------------------------------------------
// Top level DUT
//----------------------------------------------------------------
module dut( ref_clk_in,
            reset,
            clock_en_in,
            start_in,
            stop_in,
            pass_out,
            fmc_out
           );

input           ref_clk_in;
input           reset;
input           clock_en_in;
input           start_in;
input           stop_in;

output          pass_out;

output [1:0]    fmc_out;

   wire   pass_1;
   wire   pass_2;
  
   Cascaded_bram Cascaded_bram(		
                 .clk         (ref_clk_in),
		 .rst         (reset),
		 .dut_start   (start_in & (!stop_in)),
		 .dut_enable  (clock_en_in),
		 .pass        (pass_1)       
		 );
		 
   Noncascade_bram Noncascade_bram(		
                 .clk         (ref_clk_in),
		 .rst         (reset),
		 .dut_start   (start_in & (!stop_in)),
		 .dut_enable  (clock_en_in),
		 .pass        (pass_2)
		 );		 

    assign pass_out = pass_1 & pass_2;
    assign fmc_out  = {pass_1, pass_2};

endmodule // dut

//<copyright-disclaimer-start>
//<copyright-disclaimer-start>
//  **************************************************************************************************************
//  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
//  * DISCLAIMER                                                                                                 *
//  * The information contained herein is for informational purposes only, and is subject to change              *
//  * without notice. While every precaution has been taken in the preparation of this document, it              *
//  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
//  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
//  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
//  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
//  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
//  * hardware, software or other products described herein.  No license, including implied or                   *
//  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
//  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
//  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
//  *                                                                                                            *
//  **************************************************************************************************************
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
//<copyright-disclaimer-end>
