###########################################################################
## Clocking Resources Completer Script
###########################################################################

# load the standard helper file
source -quiet ../../fpgaSupport_scripts/script1.tcl
source -quiet ../../fpgaSupport_scripts/script2.tcl
# project constants
set verbose 	1
set tcName 	us_clk_resources
set demoOrLab 	completed
set projName 	wave_gen

 ## *********** Step 1 : Copying the project ***********
proc copyProject {} {

variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Set variables for source and destination paths
set sourcePath $TRAINING_PATH/$tcName/lab/$platform
set destPath $TRAINING_PATH/$tcName/completed/$platform

# Create the destination directories if they don't exist
file mkdir $destPath

# Copy the zip file to the destination directory
file copy -force $sourcePath/$language.zip $destPath
puts "sourcePath: $sourcePath"
puts "destPath: $destPath"

exec unzip $TRAINING_PATH/$tcName/completed/$platform/$language.zip -d $TRAINING_PATH/$tcName/completed/$platform/$language
}
 ## *********** Step 2 : Opening a project, Adding clk_core ***********

proc openProject {} {
variable platform
variable language
variable tcName
variable demoOrLab 
variable projName 
variable TRAINING_PATH

# Add the platform and language combination that you want 
set isLangNotSelected [string compare -nocase $language "undefined"]
set isPlatNotSelected [string compare -nocase $platform "undefined"]
   
if {$isLangNotSelected} {
      puts "Please type: use VHDL | Verilog"
      puts "   then rerun the projectCreate"
}
if {$isPlatNotSelected} {
      puts "Please type: use KCU105 | KC705 | KC7xx"
      puts "   then rerun the projectCreate"
}

# Open a project
set projName.xpr {append $projName .xpr}
open_project $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.xpr

set projName.srcs {append $projName .srcs}
create_ip -name clk_wiz -vendor xilinx.com -library ip -version 6.0 -module_name clk_core -dir $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/sources_1/ip

set_property -dict [list CONFIG.PRIM_SOURCE {Differential_clock_capable_pin} CONFIG.PRIM_IN_FREQ {300.000} CONFIG.CLKOUT2_USED {true} CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {200.000} CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {193.750} CONFIG.CLKOUT1_DRIVES {BUFG} CONFIG.CLKOUT2_DRIVES {No_buffer} CONFIG.CLKIN1_JITTER_PS {33.330000000000005} CONFIG.FEEDBACK_SOURCE {FDBK_AUTO} CONFIG.MMCM_DIVCLK_DIVIDE {3} CONFIG.MMCM_CLKFBOUT_MULT_F {7.750} CONFIG.MMCM_CLKIN1_PERIOD {3.333} CONFIG.MMCM_CLKIN2_PERIOD {10.0} CONFIG.MMCM_CLKOUT0_DIVIDE_F {3.875} CONFIG.MMCM_CLKOUT1_DIVIDE {4} CONFIG.NUM_OUT_CLKS {2} CONFIG.CLKOUT1_JITTER {121.978} CONFIG.CLKOUT1_PHASE_ERROR {113.618} CONFIG.CLKOUT2_JITTER {122.721} CONFIG.CLKOUT2_PHASE_ERROR {113.618}] [get_ips clk_core]
generate_target {instantiation_template} [get_files $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/sources_1/ip/clk_core/clk_core.xci]
generate_target all [get_files  $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/sources_1/ip/clk_core/clk_core.xci]

#launch_runs  clk_core_synth_1
}


 ## *********** Step 3 : Instantiate the clk_core or the buffer ***********

proc copyCompletedSourceCode {} {
	variable platform
	variable language
	variable tcName
	variable demoOrLab 
	variable projName 
	variable fileName
	variable TRAINING_PATH

	set completedfilePath $TRAINING_PATH/$tcName/support/completed_files
    	set projName.srcs {append $projName .srcs}
	set path $TRAINING_PATH/$tcName/$demoOrLab/$platform/$language/$projName.srcs/sources_1/imports
	
	# load all the completed file as the working file
	
	if {$language == "vhdl"} { 
		file copy -force $completedfilePath/clk_gen_us_clk_resources.vhd $path/clk_gen_us_clk_resources.vhd
	} elseif {$language == "verilog"} { 
		file copy -force $completedfilePath/clk_gen_us_clk_resources.v $path/clk_gen_us_clk_resources.v
	}
}


 ## *********** Step 4 : Implement and verify the design ***********
 ## *********** Step 5 : Examine the clocking resources ***********

proc impl_genReport {} {

# Calling the proc which runs implementation
implementationRun

report_utilization -name utilization_1
}


 ## ********** Running only the steps that are required with Make **************

proc make {stopAt} {

   puts "Running until the step $stopAt"
   set limit [string tolower $stopAt]
   switch $limit {
      step1  { copyProject }
      step2  { make step1; openProject }
      step3  { make step2; copyCompletedSourceCode }
      step4  { make step3; impl_genReport }
      all    { make step4 }
      default { 
         puts "Call the make proc, Should be make step*" 
			  }	
	}	
}

#<copyright-disclaimer-start>
#<copyright-disclaimer-start>
#  **************************************************************************************************************
#  * © 2025 Advanced Micro Devices, Inc. All rights reserved.                                                   *
#  * DISCLAIMER                                                                                                 *
#  * The information contained herein is for informational purposes only, and is subject to change              *
#  * without notice. While every precaution has been taken in the preparation of this document, it              *
#  * may contain technical inaccuracies, omissions and typographical errors, and AMD is under no                *
#  * obligation to update or otherwise correct this information.  Advanced Micro Devices, Inc. makes            *
#  * no representations or warranties with respect to the accuracy or completeness of the contents of           *
#  * this document, and assumes no liability of any kind, including the implied warranties of noninfringement,  *
#  * merchantability or fitness for particular purposes, with respect to the operation or use of AMD            *
#  * hardware, software or other products described herein.  No license, including implied or                   *
#  * arising by estoppel, to any intellectual property rights is granted by this document.  Terms and           *
#  * limitations applicable to the purchase or use of AMD’s products are as set forth in a signed agreement     *
#  * between the parties or in AMD's Standard Terms and Conditions of Sale. GD-18                               *
#  *                                                                                                            *
#  **************************************************************************************************************
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>
#<copyright-disclaimer-end>###########################################################################
## Clocking Resources Completer Script
###########################################################################

